#!/usr/bin/env python3


# The `Kubelogin` class in Python defines methods for setting up different authentication flows with
# specific parameters for obtaining tokens in Kubernetes environments.
# structure source: https://azure.github.io/kubelogin/cli/get-token.html
class Kubelogin:
    def __init__(self):
        """
        The function initializes a dictionary with specific key-value pairs related to a Kubernetes login
        command.
        """
        self.data = dict()
        self.data["exec"] = dict()
        self.data["exec"]["command"] = "kubelogin"
        self.data["exec"]["apiVersion"] = "client.authentication.k8s.io/v1beta1"
        self.data["exec"][
            "installHint"
        ] = """
        kubelogin is not installed which is required to connect to AAD enabled cluster.
        
        To learn more, please go to https://aka.ms/aks/kubelogin
        """
        self.data["exec"]["provideClusterInfo"] = False

    def NewDeviceCodeFlow(
        self, environment: str, server_id: str, client_id: str, tenant_id: str
    ):
        """
        The function `NewDeviceCodeFlow` sets up arguments for getting a token with specific environment,
        server ID, client ID, and tenant ID.

        :param environment: The `environment` parameter in the `NewDeviceCodeFlow` function likely refers to
        the environment in which the device code flow is being initiated. This could be a development,
        testing, staging, or production environment, depending on the context in which the function is being
        used
        :type environment: str
        :param server_id: The `server_id` parameter typically refers to the identifier of the server or
        resource that the code is interacting with. It is used to specify the target server or resource for
        the operation being performed
        :type server_id: str
        :param client_id: Client ID is a unique identifier assigned to the client application that is
        requesting access to a resource server. It is typically obtained when registering the client
        application with an identity provider or authorization server
        :type client_id: str
        :param tenant_id: The `NewDeviceCodeFlow` function seems to be setting up some data for a
        command-line operation related to getting a token. The parameters passed to the function are
        `environment`, `server_id`, `client_id`, and `tenant_id`
        :type tenant_id: str
        """
        self.data["exec"]["args"] = [
            "get-token",
            "--environment",
            environment,
            "--server-id",
            server_id,
            "--client-id",
            client_id,
            "--tenant-id",
            tenant_id,
        ]

    def NewWebBrowserFlow(
        self, environment: str, server_id: str, client_id: str, tenant_id: str
    ):
        """
        The function `NewWebBrowserFlow` sets up arguments for obtaining a token using interactive login in
        a web browser.

        :param environment: The `environment` parameter typically refers to the specific environment or
        instance of the web application that the code is interacting with. This could be a development,
        staging, or production environment, for example. It is used to specify where the authentication
        token should be obtained from
        :type environment: str
        :param server_id: The `server_id` parameter typically refers to the identifier of the server or
        resource that the web browser flow is interacting with. It is used to specify the target server or
        resource for authentication and authorization purposes
        :type server_id: str
        :param client_id: The `client_id` parameter is typically a unique identifier assigned to the client
        application by the authorization server when the client is registered. It is used to identify the
        client when making requests to the authorization server for authentication and authorization
        purposes
        :type client_id: str
        :param tenant_id: The `tenant_id` parameter typically refers to the unique identifier for a tenant
        within a multi-tenant application or system. It is used to distinguish between different tenants who
        are using the same application or service. Each tenant would have its own separate data,
        configurations, and resources within the shared application environment
        :type tenant_id: str
        """
        self.data["exec"]["args"] = [
            "get-token",
            "--login",
            "interactive",
            "--environment",
            environment,
            "--server-id",
            server_id,
            "--client-id",
            client_id,
            "--tenant-id",
            tenant_id,
        ]

    def NewSpSecretFlow(
        self,
        environment: str,
        server_id: str,
        client_id: str,
        tenant_id: str,
        client_secret: str,
    ):
        """
        The function `NewSpSecretFlow` sets up a data structure with specific arguments for obtaining a
        token using a service principal secret flow.

        :param environment: The `NewSpSecretFlow` function seems to be setting up a secret flow for a
        service principal in a specific environment. The function takes the following parameters:
        :type environment: str
        :param server_id: The `server_id` parameter typically refers to the identifier of the server or
        service that the code is interacting with. It is used to uniquely identify the server within the
        context of the application or system. This identifier is often required when authenticating or
        authorizing access to the server
        :type server_id: str
        :param client_id: The `client_id` parameter is typically a unique identifier assigned to a client
        application when it is registered with an identity provider or authentication service. It is used to
        identify the client application when making requests for authentication tokens or accessing
        resources
        :type client_id: str
        :param tenant_id: The `tenant_id` parameter in the `NewSpSecretFlow` function refers to the unique
        identifier of the tenant (organization) for which the client application is requesting access. This
        identifier is used in the authentication process to ensure that the client is authorized to access
        resources on behalf of that specific tenant
        :type tenant_id: str
        :param client_secret: The `NewSpSecretFlow` function seems to be setting up a secret flow for a
        service principal in a specific environment. The `client_secret` parameter is a sensitive piece of
        information that should be kept confidential. It is typically a long string of characters that
        serves as a password for the client application
        :type client_secret: str
        """
        self.data["exec"]["env"] = None
        self.data["exec"]["args"] = [
            "get-token",
            "--login",
            "spn",
            "--environment",
            environment,
            "--server-id",
            server_id,
            "--client-id",
            client_id,
            "--client-secret",
            client_secret,
            "--tenant-id",
            tenant_id,
        ]

    def NewSpPFXFlow(
        self,
        environment: str,
        server_id: str,
        client_id: str,
        tenant_id: str,
        certificate_path: str,
    ):
        """
        The function `NewSpPFXFlow` sets up a new service principal flow with specified parameters.

        :param environment: The `NewSpPFXFlow` function seems to be setting up a new service principal flow
        with the provided parameters. The `environment` parameter likely refers to the environment in which
        the service principal will be used, such as "production", "development", etc
        :type environment: str
        :param server_id: The `server_id` parameter in the `NewSpPFXFlow` method likely refers to the
        identifier or name of the server for which the token is being requested. It is used as part of the
        arguments passed to the command line tool or function to specify the server ID
        :type server_id: str
        :param client_id: The `client_id` parameter in the `NewSpPFXFlow` method refers to the unique
        identifier assigned to the client application that is requesting access to a resource. It is
        typically used in authentication and authorization processes to identify the client application
        :type client_id: str
        :param tenant_id: The `tenant_id` parameter in the `NewSpPFXFlow` method refers to the unique
        identifier of the tenant in Azure Active Directory (AAD) to which the service principal belongs.
        This identifier is used to specify the AAD tenant when authenticating and authorizing access to
        resources
        :type tenant_id: str
        :param certificate_path: The `NewSpPFXFlow` method seems to be setting up a new service principal
        flow with the provided parameters. The `certificate_path` parameter likely refers to the path where
        the client certificate is stored on the file system. This certificate is used for authentication in
        the service principal flow
        :type certificate_path: str
        """
        self.data["exec"]["env"] = None
        self.data["exec"]["args"] = [
            "get-token",
            "--login",
            "spn",
            "--environment",
            environment,
            "--server-id",
            server_id,
            "--client-id",
            client_id,
            "--client-certificate",
            certificate_path,
            "--tenant-id",
            tenant_id,
        ]

    def NewMSIFlow(self, server_id: str):
        """
        The function `NewMSIFlow` sets the arguments for retrieving a token using Managed Service Identity
        (MSI) with a specified server ID.

        :param server_id: The `server_id` parameter in the `NewMSIFlow` method is a string that represents
        the ID of the server for which a token is being requested
        :type server_id: str
        """
        self.data["exec"]["args"] = [
            "get-token",
            "--login",
            "msi",
            "--server-id",
            server_id,
        ]

    def NewMSIClientIdFlow(self, server_id: str, client_id: str):
        """
        This function sets up a new MSI client ID flow with the specified server ID and client ID.

        :param server_id: The `server_id` parameter is a string that represents the identifier of the server
        for which the token is being requested in the `NewMSIClientIdFlow` method
        :type server_id: str
        :param client_id: The `client_id` parameter is a unique identifier assigned to the client
        application that is requesting access to a resource server. It is typically used in authentication
        and authorization processes to verify the identity of the client application
        :type client_id: str
        """
        self.data["exec"]["args"] = [
            "get-token",
            "--login",
            "msi",
            "--server-id",
            server_id,
            "--client-id",
            client_id,
        ]

    def NewAzureCliFlow(self, server_id: str):
        """
        This function sets up a new Azure CLI flow with a specified server ID to get a token for
        authentication.

        :param server_id: The `NewAzureCliFlow` function seems to be setting up a flow for executing a
        specific command in the Azure CLI. The `server_id` parameter is likely used to specify the server ID
        for which the token needs to be retrieved. This server ID will be passed as an argument to the Azure
        :type server_id: str
        """
        self.data["exec"]["env"] = None
        self.data["exec"]["args"] = [
            "get-token",
            "--login",
            "azurecli",
            "--server-id",
            server_id,
        ]

    def NewWorkLoadIdentityFlow(self, server_id: str):
        """
        The function `NewWorkLoadIdentityFlow` sets the `env` key to `None` and the `args` key to a list of
        arguments for getting a token with workload identity using the provided server ID.

        :param server_id: The `server_id` parameter is a string that represents the unique identifier of a
        server. It is used in the `NewWorkLoadIdentityFlow` method to set the `server_id` value in the
        `args` dictionary key of the `data` attribute
        :type server_id: str
        """
        self.data["exec"]["env"] = None
        self.data["exec"]["args"] = [
            "get-token",
            "--login",
            "workloadidentity",
            "--server-id",
            server_id,
        ]


# The `Kubelogin` class in Python defines methods for classic kubeconfig login with specific parameters
class ClassicLogin:
    def __init__(self, token: str, client_certificate_data: str, client_key_data: str):
        """
        The function initializes a dictionary with token, client certificate data, and client key data as
        key-value pairs.

        :param token: The `token` parameter is a string that represents an authentication token used for
        authorization purposes in a client-server communication. It is typically a unique identifier that
        grants access to certain resources or services
        :type token: str
        :param client_certificate_data: The `client_certificate_data` parameter in the `__init__` method is
        a string that represents the client certificate data. This data is typically used for client
        authentication in secure communication protocols like HTTPS. It contains information about the
        client's identity and public key
        :type client_certificate_data: str
        :param client_key_data: The `client_key_data` parameter in the `__init__` method is used to store
        the client key data in the `self.data` dictionary. This data could be related to client
        authentication or encryption in a client-server communication setup. It is typically used along with
        the `client_certificate_data`
        :type client_key_data: str
        """
        self.data = dict()
        self.data["token"] = token
        self.data["client-certificate-data"] = client_certificate_data
        self.data["client-key-data"] = client_key_data



#!/usr/bin/env python3
import argparse
import json
import yaml


# get the next item in a list after a string match
def nextListItem(lst: list, item: str) -> str:
    """
    This Python function `nextListItem` takes a list and an item as input, and returns the next item in
    the list after the specified item, or None if the item is not found or is the last item in the list.

    :param lst: A list of items in which you want to find the next item after the specified item
    :type lst: list
    :param item: The `item` parameter in the `nextListItem` function is the element in the list `lst`
    for which you want to find the next element in the list
    :type item: str
    :return: The function `nextListItem` is returning the item in the list `lst` that comes immediately
    after the specified `item`. If the specified `item` is not found in the list or if it is the last
    item in the list, the function returns `None`.
    """
    try:
        return lst[lst.index(item) + 1]
    except (IndexError, ValueError):
        return None  # type: ignore


def pluckAuth(d: dict) -> list:
    """
    This Python function pluckAuth takes a dictionary as input and extracts specific keys from it,
    returning a list of corresponding values.

    :param d: A dictionary containing information related to authentication, such as environment,
    apiserver-id, client-id, tenant-id, client-secret, and certificate-path
    :type d: dict
    :return: The `pluckAuth` function returns a list containing the values corresponding to the keys
    "environment", "apiserver-id", "client-id", "tenant-id", "client-secret", and "certificate-path"
    from the input dictionary `d`. If a key is not present in the dictionary, `None` is appended to the
    list instead.
    """
    ans = []
    keys = [
        "environment",
        "apiserver-id",
        "client-id",
        "tenant-id",
        "client-secret",
        "certificate-path",
    ]
    for k in keys:
        ans.append(d.get(k, None))
    return ans


def getNext(lst: list, *args) -> list:
    """
    The function `getNext` takes a list and variable number of arguments, and returns a list of the next
    items in the list corresponding to the provided arguments.

    :param lst: The `lst` parameter in the `getNext` function is expected to be a list
    :type lst: list
    :return: The function `getNext` is returning a list of values obtained by calling the function
    `nextListItem(lst, val)` on each value passed as arguments after the initial list `lst`.
    """
    ans = []
    for _, val in enumerate(args):
        ans.append(nextListItem(lst, val))
    return ans


def overrideDefaults(args: argparse.Namespace, **kwargs) -> list:
    """
    The function `overrideDefaults` takes in arguments and keyword arguments, and returns a list of
    values based on the keyword arguments or default values from the arguments.

    :param args: argparse.Namespace object containing parsed arguments from the command line
    :type args: argparse.Namespace
    """
    ans = []
    for var, val in kwargs.items():
        if val:
            ans.append(val)
        else:
            ans.append(getattr(args, var))
    return ans


# merge multiple lists
def merge(*lists: list) -> list:
    """
    The function `merge` takes in multiple lists and non-list elements, removes duplicates, and returns
    a new list with unique elements from all inputs.

    :param : The `merge` function takes in multiple lists as arguments and combines them into a single
    list, removing any duplicate elements. If a non-list argument is provided, it will be added to the
    final list if it is not already present
    :type : list
    :return: The `merge` function returns a new list that contains all unique elements from the input
    lists.
    """
    new = []
    for l in lists:
        if isinstance(l, list):
            for i in l:
                if i not in new:
                    new.append(i)
        else:
            if l not in new:
                new.append(l)
    return new


# flatten and merge multiple lists
def flatten(*lists: list) -> list:
    """
    The `flatten` function takes in multiple lists as arguments and returns a single flattened list by
    recursively merging nested lists.

    :param : It looks like you have a function named `flatten` that takes in variable number of lists as
    arguments. The function aims to flatten nested lists into a single list. The function checks the
    type of elements in the input lists and raises an exception if it encounters an invalid type
    :type : list
    :return: The `flatten` function is returning the result of calling the `merge` function with the
    unpacked elements of the input lists.
    """
    if lists == []:
        return lists  # type: ignore
    if isinstance(lists[0], list):
        for l in lists:
            if isinstance(l[0], list):
                return flatten(merge(*l))
            if isinstance(l[0], str) or isinstance(l[0], int) or isinstance(l[0], dict):
                pass
            else:
                raise Exception(TypeError("Invalid type in list"))
    return merge(*lists)  # type: ignore


# write to a file
def write(file: str, data: str) -> bool:
    """
    The function `write` takes a file path and data as input, attempts to write the data to the file,
    and returns True if successful or False if an error occurs.

    :param file: The `file` parameter in the `write` function is a string that represents the file path
    where the data will be written to
    :type file: str
    :param data: The `data` parameter in the `write` function is a string that represents the content
    that you want to write to the file specified by the `file` parameter. When you call the `write`
    function, you provide the content you want to write to the file as the `data` argument
    :type data: str
    :return: The function `write` returns a boolean value - `True` if the data was successfully written
    to the file, and `False` if an error occurred during the writing process.
    """
    try:
        with open(file, "w") as f:
            f.write(data)
    except Exception as e:
        print(f"Error: {e}")
        return False
    return True


def determine_auth_flow(args: argparse.Namespace) -> str:
    """
    This Python function determines the authentication flow based on the arguments provided.

    :param args: The `determine_auth_flow` function takes in a namespace object `args` as input and
    determines the authentication flow based on the attributes of the `args` object. The function checks
    various attributes of the `args` object to determine the authentication flow and returns a string
    indicating the type of authentication flow
    :type args: argparse.Namespace
    :return: The function `determine_auth_flow` is returning a string based on the arguments provided.
    The string returned depends on the specific argument that is set to True. If none of the arguments
    are True, the function will return "default".
    """
    if args.default:
        return "default"
    if args.device_flow:
        return "device_flow"
    if args.interactive:
        return "interactive"
    if args.sp_secret:
        return "sp_secret"
    if args.sp_pfx:
        return "sp_pfx"
    if args.managed_identity:
        return "managed_identity"
    if args.managed_identity_id:
        return "managed_identity_id"
    if args.az_cli:
        return "az_cli"
    if args.workload_identity:
        return "workload_identity"
    return "default"


def print_json(clusters: list, **kwargs) -> bool:
    """
    The function `print_json` takes a list of clusters and optional keyword arguments, attempts to
    convert the data to JSON format, and either prints the JSON output or writes it to a file if an
    outfile is provided.

    :param clusters: The `clusters` parameter is expected to be a list that contains the data you want
    to convert to JSON format. This function will attempt to convert this list into a JSON string with
    an indentation of 2 spaces. If successful, it will either print the JSON string to the console or
    write it to
    :type clusters: list
    :return: The function `print_json` returns a boolean value - `True` if the JSON data was
    successfully printed or written to a file, and `False` if there was an error during the process.
    """
    outfile = kwargs.get("outfile", None)
    try:
        # attempt to marshal the data to JSON
        j = json.dumps(clusters, indent=2)
    except Exception as e:
        print(f"Error: {e}")
        return False
    if not outfile:
        print(j)
    # Write to a file if an outfile is provided
    if outfile:
        write(outfile, j)
    return True


def print_yaml(clusters: list, **kwargs) -> bool:
    """
    The function `print_yaml` takes a list of clusters and optional keyword arguments, attempts to
    convert the data to YAML format, and either prints the YAML output or writes it to a file based on
    the provided `outfile` argument.

    :param clusters: The `clusters` parameter is expected to be a list of data that you want to convert
    to YAML format. This function `print_yaml` takes this list as input and converts it to a YAML format
    string. If successful, it either prints the YAML string to the console or writes it to a file
    :type clusters: list
    :return: The function `print_yaml` returns a boolean value - `True` if the data was successfully
    marshaled to YAML and written to a file (if an outfile is provided), and `False` if there was an
    error during the marshaling process.
    """
    outfile = kwargs.get("outfile", None)
    try:
        # attempt to marshal the data to YAML
        y = yaml.dump(clusters)
    except Exception as e:
        print(f"Error: {e}")
        return False
    if not outfile:
        print(y)
    # Write to a file if an outfile is provided
    if outfile:
        write(outfile, y)
    return True
